<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink"
 xmlns:math="http://exslt.org/math"
 extension-element-prefixes="math">


    <xsl:import href="common-scripts.xsl"/>
    <xsl:import href="../../util/exslt/math.constants.template.xsl"/>
    <xsl:import href="../../util/exslt/math.sine.template.xsl"/>
    <xsl:import href="../../util/exslt/math.cosine.template.xsl"/>

    <xsl:variable name="graph-radius" select="0.42"/>
    <xsl:variable name="max-value-radius" select="0.40"/>

<xsl:template name="spider-graph">
    <xsl:param name="title"/>
  	<xsl:param name="width"/>
  	<xsl:param name="height"/>
    <xsl:param name="metrics-nodeset"/>

    <xsl:variable   name="color-ok"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
    <xsl:variable   name="color-warn"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
    <xsl:variable   name="color-ko"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>


    <svg viewBox="0 0 {$width} {$height}" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">

        <xsl:variable name="metrics" select="xalan:nodeset($metrics-nodeset)"/>

        <xsl:call-template name="common-scripts"/>

        <!-- Show outline of canvas using 'rect' element -->
        <rect x="1" y="1" fill="none" stroke-width="1">
            <xsl:attribute name="stroke"><xsl:value-of select="$color-ko"/></xsl:attribute>
            <xsl:attribute name="width"><xsl:value-of select="$width - 2"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="$height - 2"/></xsl:attribute>
        </rect>

        <xsl:apply-templates select="$metrics/metrics" mode="draw-web">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
        </xsl:apply-templates> '

        <xsl:call-template name="legend">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
        </xsl:call-template>

    </svg>
</xsl:template>

<xsl:template match="metrics" mode="draw-web">
    <xsl:param name="width"/>
    <xsl:param name="height"/>

    <xsl:variable name="metrics-number" select="count(metric)"/>
    <xsl:variable name="metrics" select="current()"/>

    <xsl:variable name="center-x" select="$width * 0.5"/>
    <xsl:variable name="center-y" select="$height * 0.5"/>

    <!-- center cicle -->
    <circle
        style="fill:black;stroke:black;stroke-width:1;">
        <xsl:attribute name="r">
            <xsl:value-of select="0.005*$width"/>
        </xsl:attribute>
        <xsl:attribute name="cx">
            <xsl:value-of select="$center-x"/>
        </xsl:attribute>
        <xsl:attribute name="cy">
            <xsl:value-of select="$center-x"/>
        </xsl:attribute>
    </circle>

    <xsl:for-each select="metric">
        <xsl:call-template name="spider-lines">
            <xsl:with-param name="position" select="position()"/>
            <xsl:with-param name="metrics" select="$metrics"/>
            <xsl:with-param name="metrics-number" select="$metrics-number"/>
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
        </xsl:call-template>
    </xsl:for-each>
</xsl:template>

<xsl:template name="spider-lines">

    <xsl:param name="position"/>
    <xsl:param name="metrics"/>
    <xsl:param name="metrics-number"/>
    <xsl:param name="width"/>
    <xsl:param name="height"/>


    <xsl:variable   name="color-ok"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
    <xsl:variable   name="color-warn"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
    <xsl:variable   name="color-ko"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>

    <xsl:variable name="center-x" select="$width * 0.5"/>
    <xsl:variable name="center-y" select="$height * 0.5"/>

    <xsl:variable name="this-rad" select="2 * $math:pi * ($position -1) div ($metrics-number)"/>
    <xsl:variable name="this-sine-rad">
        <xsl:call-template name="math:sine">
            <xsl:with-param name="base" select="$this-rad"/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="this-cosine-rad">
        <xsl:call-template name="math:cosine">
            <xsl:with-param name="base" select="$this-rad"/>
        </xsl:call-template>
    </xsl:variable>


    <xsl:variable name="prev-rad">
        <xsl:choose>
            <xsl:when test="$position &gt; 1">
                <xsl:value-of select="2 * $math:pi * ($position -2) div ($metrics-number)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="2 * $math:pi * ($metrics-number -1) div ($metrics-number)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="prev-sine-rad">
        <xsl:call-template name="math:sine">
            <xsl:with-param name="base" select="$prev-rad"/>
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="prev-cosine-rad">
        <xsl:call-template name="math:cosine">
            <xsl:with-param name="base" select="$prev-rad"/>
        </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="this-metric" select="$metrics/metric[$position]"/>

    <xsl:call-template name="radial-lines">
        <xsl:with-param name="width" select="$width"/>
        <xsl:with-param name="height" select="$height"/>
        <xsl:with-param name="center-x" select="$center-x"/>
        <xsl:with-param name="center-y" select="$center-y"/>
        <xsl:with-param name="this-sine-rad" select="$this-sine-rad"/>
        <xsl:with-param name="this-cosine-rad" select="$this-cosine-rad"/>
    </xsl:call-template>


    <xsl:call-template name="max-circle-lines">
        <xsl:with-param name="width" select="$width"/>
        <xsl:with-param name="height" select="$height"/>
        <xsl:with-param name="center-x" select="$center-x"/>
        <xsl:with-param name="center-y" select="$center-y"/>
        <xsl:with-param name="this-sine-rad" select="$this-sine-rad"/>
        <xsl:with-param name="this-cosine-rad" select="$this-cosine-rad"/>
        <xsl:with-param name="prev-sine-rad" select="$prev-sine-rad"/>
        <xsl:with-param name="prev-cosine-rad" select="$prev-cosine-rad"/>
    </xsl:call-template>


    <xsl:call-template name="median-circle-lines">
        <xsl:with-param name="width" select="$width"/>
        <xsl:with-param name="height" select="$height"/>
        <xsl:with-param name="center-x" select="$center-x"/>
        <xsl:with-param name="center-y" select="$center-y"/>
        <xsl:with-param name="this-sine-rad" select="$this-sine-rad"/>
        <xsl:with-param name="this-cosine-rad" select="$this-cosine-rad"/>
        <xsl:with-param name="prev-sine-rad" select="$prev-sine-rad"/>
        <xsl:with-param name="prev-cosine-rad" select="$prev-cosine-rad"/>
        <xsl:with-param name="position" select="$position"/>
        <xsl:with-param name="metrics" select="$metrics"/>
        <xsl:with-param name="metrics-number" select="$metrics-number"/>
    </xsl:call-template>

    <xsl:call-template name="actual-value-circle-lines">
        <xsl:with-param name="width" select="$width"/>
        <xsl:with-param name="height" select="$height"/>
        <xsl:with-param name="center-x" select="$center-x"/>
        <xsl:with-param name="center-y" select="$center-y"/>
        <xsl:with-param name="this-sine-rad" select="$this-sine-rad"/>
        <xsl:with-param name="this-cosine-rad" select="$this-cosine-rad"/>
        <xsl:with-param name="prev-sine-rad" select="$prev-sine-rad"/>
        <xsl:with-param name="prev-cosine-rad" select="$prev-cosine-rad"/>
        <xsl:with-param name="position" select="$position"/>
        <xsl:with-param name="metrics" select="$metrics"/>
        <xsl:with-param name="metrics-number" select="$metrics-number"/>
    </xsl:call-template>

	<xsl:variable name="text-x-pos" select="$center-x + $width * 0.45 * $this-cosine-rad"/>
	<xsl:variable name="text-y-pos" select="$center-y - $height * 0.45 * $this-sine-rad"/>


    <text font-family="Verdana" x="{$text-x-pos}" y="{$text-y-pos}">
        <xsl:attribute name="fill"><xsl:value-of select="$color-ok"/></xsl:attribute>
        <xsl:attribute name="font-size">
            <xsl:value-of select="ceiling($width * 0.02)"/>
        </xsl:attribute>
        <xsl:choose>
        	<xsl:when test="$text-x-pos &lt; 0.20 * $center-x">
        		<xsl:attribute name="style">text-anchor:start;</xsl:attribute>
            </xsl:when>
            <xsl:when test="$text-x-pos &gt; 0.80 * $center-x">
        		<xsl:attribute name="style">text-anchor:end;</xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
        		<xsl:attribute name="style">text-anchor:middle;</xsl:attribute>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="$this-metric/@name"/>
	</text>


</xsl:template>

<xsl:template name="radial-lines">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="center-x"/>
    <xsl:param name="center-y"/>
    <xsl:param name="this-sine-rad"/>
    <xsl:param name="this-cosine-rad"/>

    <xsl:variable name="this-x" select="$center-x + $width * $graph-radius * $this-cosine-rad"/>
    <xsl:variable name="this-y" select="$center-y - $height * $graph-radius * $this-sine-rad"/>

    <line style="fill:none;stroke:black;stroke-width:1;">
    	<xsl:attribute name="x1"><xsl:value-of select="$center-x"/></xsl:attribute>
    	<xsl:attribute name="x2"><xsl:value-of select="$this-x"/></xsl:attribute>
    	<xsl:attribute name="y1"><xsl:value-of select="$center-y"/></xsl:attribute>
    	<xsl:attribute name="y2"><xsl:value-of select="$this-y"/></xsl:attribute>
    </line>
</xsl:template>

<xsl:template name="max-circle-lines">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="center-x"/>
    <xsl:param name="center-y"/>
    <xsl:param name="this-sine-rad"/>
    <xsl:param name="this-cosine-rad"/>
    <xsl:param name="prev-sine-rad"/>
    <xsl:param name="prev-cosine-rad"/>

    <xsl:variable name="this-x" select="$center-x + $width * $max-value-radius * $this-cosine-rad"/>
    <xsl:variable name="this-y" select="$center-y - $height * $max-value-radius * $this-sine-rad"/>
    <xsl:variable name="prev-x" select="$center-x + $width * $max-value-radius * $prev-cosine-rad"/>
    <xsl:variable name="prev-y" select="$center-y - $height * $max-value-radius * $prev-sine-rad"/>

    <line style="fill:none;stroke:black;stroke-width:1;">
        <xsl:attribute name="x1"><xsl:value-of select="$prev-x"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$this-x"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$prev-y"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$this-y"/></xsl:attribute>
    </line>
</xsl:template>

<xsl:template name="median-circle-lines">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="center-x"/>
    <xsl:param name="center-y"/>
    <xsl:param name="this-sine-rad"/>
    <xsl:param name="this-cosine-rad"/>
    <xsl:param name="prev-sine-rad"/>
    <xsl:param name="prev-cosine-rad"/>
    <xsl:param name="position"/>
    <xsl:param name="metrics"/>
    <xsl:param name="metrics-number"/>

    <xsl:variable name="this-metric" select="$metrics/metric[$position]"/>

    <xsl:variable name="prev-metric-median-div-max">
        <xsl:choose>
            <xsl:when test="$position &gt; 1">
	        <xsl:choose>
			<xsl:when test="$metrics/metric[$position -1]/@max = 0">
				<xsl:value-of select="0.5"/>
			</xsl:when>
			<xsl:otherwise>
	        	        <xsl:value-of select="$metrics/metric[$position -1]/@median div $metrics/metric[$position -1]/@max"/>
			</xsl:otherwise>
	        </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
	        <xsl:choose>
			<xsl:when test="$metrics/metric[$metrics-number]/@max = 0">
				<xsl:value-of select="0.5"/>
			</xsl:when>
			<xsl:otherwise>
	        	        <xsl:value-of select="$metrics/metric[$metrics-number]/@median div $metrics/metric[$metrics-number]/@max"/>
			</xsl:otherwise>
	        </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="this-median-div-max">
	<xsl:choose>
		<xsl:when test="$this-metric/@max = 0">
			<xsl:value-of select="0.5"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$this-metric/@median div $this-metric/@max"/>
		</xsl:otherwise>
	</xsl:choose>
    </xsl:variable>


    <xsl:variable name="this-x" select="$center-x + $width * $max-value-radius * $this-cosine-rad * ($this-median-div-max)"/>
    <xsl:variable name="this-y" select="$center-y - $height * $max-value-radius * $this-sine-rad * ($this-median-div-max)"/>
    <xsl:variable name="prev-x" select="$center-x + $width * $max-value-radius * $prev-cosine-rad * ($prev-metric-median-div-max)"/>
    <xsl:variable name="prev-y" select="$center-y - $height * $max-value-radius * $prev-sine-rad * ($prev-metric-median-div-max)"/>

    <line style="fill:none;stroke:black;stroke-dasharray:10 10;stroke-width:1;">
        <xsl:attribute name="x1"><xsl:value-of select="$prev-x"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$this-x"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$prev-y"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$this-y"/></xsl:attribute>
    </line>
</xsl:template>

<xsl:template name="actual-value-circle-lines">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="center-x"/>
    <xsl:param name="center-y"/>
    <xsl:param name="this-sine-rad"/>
    <xsl:param name="this-cosine-rad"/>
    <xsl:param name="prev-sine-rad"/>
    <xsl:param name="prev-cosine-rad"/>
    <xsl:param name="position"/>
    <xsl:param name="metrics"/>
    <xsl:param name="metrics-number"/>

    <xsl:variable   name="color-ok"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 3]/@color"/>
    <xsl:variable   name="color-warn"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 2]/@color"/>
    <xsl:variable   name="color-ko"    select="$radar-config-doc/radar-config/defined-metrics/thresholds/threshold[@level = 1]/@color"/>

    <xsl:variable name="this-metric" select="$metrics/metric[$position]"/>

    <xsl:variable name="prev-metric-value-div-max">
        <xsl:choose>
            <xsl:when test="$position &gt; 1">
	        <xsl:choose>
        	    <xsl:when test="$metrics/metric[$position -1]/@max = 0">
                	<xsl:value-of select="0"/>
	            </xsl:when>
        	    <xsl:otherwise>
                	<xsl:value-of select="$metrics/metric[$position -1]/@value div $metrics/metric[$position -1]/@max"/>
	            </xsl:otherwise>
        	</xsl:choose>
            </xsl:when>
            <xsl:otherwise>
	        <xsl:choose>
        	    <xsl:when test="$metrics/metric[$metrics-number]/@max = 0">
                	<xsl:value-of select="0"/>
	            </xsl:when>
        	    <xsl:otherwise>
                	<xsl:value-of select="$metrics/metric[$metrics-number]/@value div $metrics/metric[$metrics-number]/@max"/>
	            </xsl:otherwise>
        	</xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:variable name="this-metric-value-div-max">
	<xsl:choose>
		<xsl:when test="$this-metric/@max = 0">
			<xsl:value-of select="0"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$this-metric/@value div $this-metric/@max"/>
		</xsl:otherwise>
	</xsl:choose>
    </xsl:variable>

    <xsl:variable name="this-x" select="$center-x + $width * $max-value-radius * $this-cosine-rad * ($this-metric-value-div-max)"/>
    <xsl:variable name="this-y" select="$center-y - $height * $max-value-radius * $this-sine-rad * ($this-metric-value-div-max)"/>
    <xsl:variable name="prev-x" select="$center-x + $width * $max-value-radius * $prev-cosine-rad * ($prev-metric-value-div-max)"/>
    <xsl:variable name="prev-y" select="$center-y - $height * $max-value-radius * $prev-sine-rad * ($prev-metric-value-div-max)"/>

    <line>
        <xsl:attribute name="style">fill:none;stroke:<xsl:value-of select="$color-ko"/>;stroke:width:2;</xsl:attribute>
        <xsl:attribute name="x1"><xsl:value-of select="$prev-x"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$this-x"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$prev-y"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$this-y"/></xsl:attribute>
    </line>
</xsl:template>


<xsl:template name="legend">
    <xsl:param name="width"/>
    <xsl:param name="height"/>

    <xsl:variable name="legend-height" select="0.98"/>

    <line style="fill:none;stroke:black;stroke-width:1;">
        <xsl:attribute name="x1"><xsl:value-of select="$width * 0.10"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$width * 0.20"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
    </line>

    <text font-family="Verdana" fill="black" x="{$width * 0.21}" y="{$height * $legend-height}">
        <xsl:attribute name="font-size">
            <xsl:value-of select="ceiling($width * 0.02)"/>
        </xsl:attribute>
        <xsl:attribute name="style">text-anchor:start;</xsl:attribute>
        Max Value
    </text>

    <line style="fill:none;stroke:#BFDE1D;stroke-width:2;">
        <xsl:attribute name="x1"><xsl:value-of select="$width * 0.40"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$width * 0.50"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
    </line>

    <text font-family="Verdana" fill="black" x="{$width * 0.51}" y="{$height * $legend-height}">
        <xsl:attribute name="font-size">
            <xsl:value-of select="ceiling($width * 0.02)"/>
        </xsl:attribute>
        <xsl:attribute name="style">text-anchor:start;</xsl:attribute>
        Actual Value
    </text>

    <line style="fill:none;stroke:black;stroke-dasharray:10 10;stroke-width:1;">
        <xsl:attribute name="x1"><xsl:value-of select="$width * 0.70"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="$width * 0.80"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="$height * $legend-height"/></xsl:attribute>
    </line>

    <text font-family="Verdana" fill="black" x="{$width * 0.81}" y="{$height * $legend-height}">
        <xsl:attribute name="font-size">
            <xsl:value-of select="ceiling($width * 0.02)"/>
        </xsl:attribute>
        <xsl:attribute name="style">text-anchor:start;</xsl:attribute>
        Medium Value
    </text>
</xsl:template>


</xsl:stylesheet>

